<?php
/**
 * Application_Cli
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @package Application
 * @subpackage Cli
 * 
 */

/**
 * Command line interface for Mai. (Mai Manager)
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @package Application
 * @subpackage Cli
 * 
 *  
 */
class Application_Cli extends Application_Bootstrap_Base
{
	/**
	 * Private constructor to prevent creation of this object.
	 * @return void
	 */
	private function __construct(){}
	
	/**
	 * Starts up the bootstrap.
	 * @param $arguments
	 * @return void
	 */
	static public function run()
	{
		self::output("\n---\n Mai Command Line Interface \n\n");
		self::output(" Copyright (C) 2009  Mai (Me Artificial Intelligence) \n");
		self::output(" Licensed under the GNU General Public License version 3 (GPLv3) \n");
		self::output("---\n\n");

		do
		{
			$exit = false;
			self::output("This CLI can install, update or change your Mai installation. \n");
			self::output("Please proceed with caution! \n");
			do
			{
				self::output("\nWhat do you want to do?\n\n");
				self::output("   [1] Install \n");
				self::output("   [2] Backup database \n");
				self::output("   [q] Exit  \n\n");
				self::output("Option: ");

				$option = trim(fgets(STDIN));
			}
			while(!in_array($option, array(1,2, 'q', '[1]', '[2]', '[q]')));

			switch($option)
			{
				case 1: //Install
				case '[1]':
					self::runInstall();
					break;
				case 2:
				case '[2]':
					self::runBackupDb();
					self::output("\n\nCLI finished.\n\n");
					break;
				case 'q':
				case '[q]':
				default:
					self::output("\n\nCLI closed.");
					$exit = true;
					break;
			}

			//Ask to exit the manager
			self::output("\n\nDo you want to exit the manager? (y/n) ");
			$option = trim(fgets(STDIN));
			if($option == 'y')
			{
				$exit = true;
			}
		}
		while(!$exit);
	}
	
	/**
	 * Ask the user to confirm.
	 * @param $message
	 * @return void
	 */
	static private function askToConfirm($message = "\n\nDo you wish to continue? (y\n) ")
	{
		//Loop until the answer is 'y' or 'n'
		do
		{
			fwrite(STDOUT, $message);
			$continue = trim(fgets(STDIN));
			if($continue == 'n')
			{
				return false;
			}
		}
		while($continue != 'n' && $continue != 'y');

		return true;
	}

	/**
	 * Ask the user to continue
	 * @return void
	 */
	static private function askToContinue()
	{
		//Loop until the answer is 'y' or 'n'
		do
		{
			fwrite(STDOUT, "\n\nDo you wish to continue? (y/n) ");

			$continue = trim(fgets(STDIN));
			if($continue == 'n')
			{
				fwrite(STDOUT, "\n" . "Exiting manager..." . "\n");
				exit;
			}

			fwrite(STDOUT, "\n");
		}
		while($continue != 'n' && $continue != 'y');
	}
	
	/**
	 * Output a message.
	 * @param $message
	 * @return void
	 */
	static private function output($message)
	{
		fwrite(STDOUT, $message);
	}
	
	/**
	 * Test the PHP version.
	 * @return Boolean
	 */
	static private function testPhpVersion()
	{
		return version_compare(PHP_VERSION, '5.2.4', '>');
	}

	/**
	 * Look for a loaded database extension.
	 * @return Boolean
	 */
	static private function testDatabaseDrivers()
	{
		$loaded = false;

		if(extension_loaded('fbsql'))
		{
			self::output("FrontBase: OK\n");
			$loaded = true;
		}
		else
		{
			self::output("FrontBase: Not found!\n");
		}

		if(extension_loaded('ibase'))
		{
			self::output("InterBase / Firebird: OK\n");
			$loaded = true;
		}
		else 
		{
			self::output("InterBase / Firebird: Not found!\n");
		}

		if(extension_loaded('mssql'))
		{
			self::output("Microsoft SQL Server: OK\n");
			$loaded = true;
		}
		else
		{
			self::output("Microsoft SQL Server: Not found!\n");
		}

		if(extension_loaded('mysql'))
		{
			self::output("MySQL: OK\n");
			$loaded = true;
		}
		else
		{
			self::output("MySQL: Not found!\n");
		}

		if(extension_loaded('mysqli'))
		{
			self::output("MySQLi: OK\n");
			$loaded = true;
		}
		else
		{
			self::output("MySQLi: Not found!\n");
		}

		if(extension_loaded('oci'))
		{
			self::output("Oracle: OK\n");
			$loaded = true;
		}
		else
		{
			self::output("Oracle: Not found!\n");
		}

		if(extension_loaded('pqsql'))
		{
			self::output("PostgreSQL: OK\n");
			$loaded = true;
		}
		else
		{
			self::output("PostgreSQL: Not found!\n");
		}

		if(extension_loaded('querysim'))
		{
			self::output("QuerySim: OK\n");
			$loaded = true;
		}
		else
		{
			self::output("QuerySim: Not found!\n");
		}

		if(extension_loaded('sqlite'))
		{
			self::output("SQLite 2: OK\n");
			$loaded = true;
		}
		else
		{
			self::output("SQLite 2: Not found!\n");
		}

		return $loaded;
	}

	/**
	 * Create a config file for Mai
	 * @return void
	 */
	static private function createConfig()
	{
		$config = array();

		//Ask wich db extension to use
		self::output("\nMai creates 2 databases. Wich extension should be used? (e.g. mysql or psql) ");
		do
		{
			$extension = trim(fgets(STDIN));
			$config['db']['db_user']['dbtype'] = $extension;
			$config['db']['db_user']['connection_name'] = 'mai_userdb_connection';
			$config['db']['db_base']['dbtype'] = $extension;
			$config['db']['db_base']['connection_name'] = 'mai_basedb_connection';
		}
		while(!in_array($extension, array('mysql', 'fbsql', 'ibase', 'mssql', 'mysqli', 'oci', 'pgsql', 'querysim', 'sqlite')));

		//Ask wich host to use
		self::output("\nWhat host should be used? (default is localhost) ");
		$host = trim(fgets(STDIN));
		$config['db']['db_user']['host'] = $host;
		$config['db']['db_base']['host'] = $host;

		//Ask for dbnames
		self::output("\nWhat databse name should be used for the base database? (default is mai_base) ");
		$config['db']['db_base']['dbname'] = trim(fgets(STDIN));
		self::output("\nWhat databse name should be used for the user database? (default is mai_user) ");
		$config['db']['db_user']['dbname'] = trim(fgets(STDIN));

		//Ask for user and password (base)
		self::output("\nA database user is needed with rights to create the base database.");
		self::output("\nPlease create a database named ".$config['db']['db_base']['dbname']."and a user with rights on it.");
		self::output("\nAlternatively you can create a user wich has the right to create a database.");
		self::output("\nWhat is the username of the user for the BASE db? ");
		$config['db']['db_base']['username'] = $host = trim(fgets(STDIN));

		self::output("\nWhat is the password of the user for the BASE db? ");
		$config['db']['db_base']['password'] = $host = trim(fgets(STDIN));

		//Ask for user and password (user)
		self::output("\nA database user is needed with rights to create the user database.");
		self::output("\nPlease create a database named ".$config['db']['db_user']['dbname']."and a user with rights on it.");
		self::output("\nAlternatively you can create a user wich has the right to create a database.");
		self::output("\nWhat is the username of the user for the USER db? ");		
		$config['db']['db_user']['username'] = $host = trim(fgets(STDIN));;

		self::output("\nWhat is the password of the user for the USER db? ");
		$config['db']['db_user']['password'] = $host = trim(fgets(STDIN));

		//End config
		self::output("\nThe config file will now be created. These values can be changed by editing the config file:");
		self::output("\n" . APPLICATION_PATH . "\config\config.ini");

		//Set development to on
		$config['global']['application']['development'] = 1;

		//Open the ini file to write
		$iniFile = fopen(APPLICATION_PATH . "/config/config.ini", "w");
		do
		{
			$success = true;
			//Try to write to the file
			try
			{
				fwrite($iniFile, ";Config file created by Mai Manager at .".date("d-m-Y H:i")."\n\n");
			}
			catch(Exception $e)
			{
				$success = false;
				self::output("\n\nWARNING: " . APPLICATION_PATH . "/config/config.ini is not writable!");
				self::output("\nPlease make this file writable.");
				self::askToContinue();
			}
		}
		while($success == false);

		//Write config to file
		foreach($config as $section => $sectionContent)
		{
			//Create section
			fwrite($iniFile, "[$section]\n");

			foreach($sectionContent as $head => $sub)
			{
				//Create name of the variable (head.sub = value)
				foreach($sub as $name => $value)
				{
					fwrite($iniFile, "$head.$name = $value\n");
				}
			}
		}
	}

	/**
	 * Test the connection with the base db.
	 * @return Boolean
	 */
	static private function testBaseDb()
	{
		$config = parent::returnDbConfig();
		//Data Source Name
		$dsn =  sprintf('%s://%s:%s@%s/%s',
	                $config->db_base->dbtype,
	                $config->db_base->username,
	                $config->db_base->password,
	                $config->db_base->host,
	                $config->db_base->dbname);
		
		$conn = Doctrine_Manager::connection($dsn, $config->db_base->connection_name);
		try
		{
			//Test the connection
			$conn->beginTransaction();
			//Save
			Zend_Registry::set('conn_dbbase', $conn);
		}
		catch(Doctrine_Connection_Exception $e)
		{
			return false;
		}

		return true;
	}

	/**
	 * Test the connection with the user db.
	 * @return Boolean
	 */
	static private function testUserDb()
	{
		$config = parent::returnDbConfig();
		//Data Source Name
		$dsn =  sprintf('%s://%s:%s@%s/%s',
	                $config->db_user->dbtype,
	                $config->db_user->username,
	                $config->db_user->password,
	                $config->db_user->host,
	                $config->db_user->dbname);

		$conn = Doctrine_Manager::connection($dsn, $config->db_user->connection_name);
		
		try
		{
			//Test the connection
			$conn->beginTransaction();
			//Save
			Zend_Registry::set('conn_dbuser', $conn);
		}
		catch(Doctrine_Connection_Exception $e)
		{
			return false;
		}

		return true;
	}

	/**
	 * Install Mai
	 * @return void
	 */
	static private function runInstall()
	{
		self::output("\nChecking requirements.. ");

		//Test php version
		if(!self::testPhpVersion())
		{
			self::output("\n\nPHP 5.2.4 or higher is recommended! Your version is ".PHP_VERSION.
			"\nMai could run but this is not tested. No support is given!");

			self::askToContinue();
		}
		else
		{
			self::output("\nPHP: OK (".PHP_VERSION.") \n");
		}

		//Test database
		if(!self::testDatabaseDrivers())
		{
			self::output("\n\nNo database extension is found. Please enable or install an extension.");
			self::output("\nMai cannot run without an database! Installation aborted.");
			exit;
		}
		else
		{
			self::output("\nValid database extension found.");
		}

		self::output("\nCreating new config..");

		//Ask to create a custom config file
		if(self::askToConfirm("\n\nDo you want to configure Mai? (recommended) (y/n) "))
		{
			self::createConfig();
		}

		self::output("\nBootstrapping started..");

		//Set include paths
		parent::initIncludePaths();
		//Start auto loader
		parent::initAutoLoader();
		//Load global config
		parent::initGlobalConfig();

		self::output("\nBootstrapping completed.");
		self::output("\nTesting database connection: base");

		//Start base db connection
		if(!self::testBaseDb())
		{
			self::output("\n\nWARNING: Failed to connect to the base db.");
			self::output("\nYou can configure Mai yourself and update the database using this installer.");
			self::output("\nAlternatively you can reconfigure Mai.");
			if(self::askToConfirm("\n\nDo you want to reconfigure Mai? (recommended) (y/n) "))
			{
				self::createConfig();
			}
			else
			{
				if(!self::askToConfirm("\n\nDo you want to continue with the installation? (y/n) "))
				{
					//Exit installation
					self::output("Installation cancelled.");
					exit;
				}

			}
		}
		else
		{
			self::output("\nBase Database connection: OK");
			//Doctrine Command Line interface
			$cli = new Doctrine_Cli(parent::returnDoctrineCliConfigBase());

			if(!self::askToConfirm("\nDoes the base database already excist? (y/n) "))
			{
				//Create base db

				$cli->run(array('mai-manager', 'create-db basedb'));
			}

			//Create tables and data
			$cli->run(array('mai-manager', 'generate-sql basedb'));
			$cli->run(array('mai-manager', 'create-tables basedb'));
			//Free from memory
			unset($cli);
		}

		self::output("\nTesting database connection: user");
		//Start user db connection
		if(!self::testUserDb())
		{
			self::output("\n\nWARNING: Failed to connect to the user db.");
			self::output("\nYou can configure Mai yourself and update the database using this installer.");
			self::output("\nAlternatively you can reconfigure Mai.");
			if(self::askToConfirm("\n\nDo you want to reconfigure Mai? (recommended) (y/n) "))
			{
				self::createConfig();
			}
			else
			{
				if(!self::askToConfirm("\n\nDo you want to continue with the installation? (y/n) "))
				{
					//Exit installation
					self::output("Installation cancelled.");
					exit;
				}

			}
		}
		else
		{
			self::output("\nUser Database connection: OK");
			//Doctrine Command Line interface
			$cli = new Doctrine_Cli(parent::returnDoctrineCliConfigUser());

			if(!self::askToConfirm("\nDoes the user databse already excist? (y/n) "))
			{
				//Create base db

				$cli->run(array('mai-manager', 'create-db'));
			}

			//Create tables and data
			$cli->run(array('mai-manager', 'generate-sql'));
			$cli->run(array('mai-manager', 'create-tables'));
			//Free from memory
			unset($cli);
		}

		if(self::askToConfirm("\nDo you want to test your Mai installation? (cURL extension required) (y/n) "))
		{
			if(extension_loaded('curl'))
			{
				//Use the URL to test Mai
				self::output("\nWhat is the url of your Mai installation? (default is http://localhost) ");
				$url =  trim(fgets(STDIN));
				$test = curl_init($url . '/talk/');
				//Set to return
				curl_setopt($test, CURLOPT_RETURNTRANSFER, true);
				//Get the result
				$result = curl_exec($test);
				curl_close($test);

				//The TalkController should output invalid-session
				if($result == 'invalid-session')
				{
					self::output("\nTest: OK");
				}
				else
				{
					self::output("\nTest failed. Response:\n\n".$result);
				}
			}
			else
			{
				self::output("\nTest failed, cURL is not activated.");
			}
		}

		self::output("\n\n\nMai installation finished! Enjoy!");
	}

	/**
	 * Backup the Mai database
	 * @return void
	 */
	static private function runBackupDb()
	{
		//@Mai Todo: Backup function not yet finished.
		self::output("\n\nMai Manager is not yet able to backup your database.");
	}

}
