<?php
/**
 * TalkController
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @package Controllers
 * @subpackage Ai
 * 
 */

/**
 * TalkController
 * 
 * Possible responses:
 * 
 * [1] 'invalid-session' When no session excists.
 * [2] 'invalid-input' When the input is invalid or not set.
 * [3] 'no-message' No message set
 * [4] '...' A response from Mai
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @package Controllers
 * @subpackage Ai
 * 
 * 
 */
 
class TalkController extends Zend_Controller_Action
{
	/**
	 * Combines the different components of Mai Lib to parse the text of the user.
	 */
	public function indexAction()
    {
    	//Start up Mai Library
    	$this->startUpMai();
    	
 		//Check the user session
    	if(!User_Session::isValid())
		{
			Mai_Registry::set('output', Mai_Config::getVar()->output->invalid_session);
		}
		else
		{		
	    	//Create a filter
	    	$filterChain = new Zend_Filter();
	    	
	    	$filterChain->addFilter(new Zend_Filter_StringToLower())
	    				->addFilter(new Zend_Filter_StringTrim())
	    				->addFilter(new Zend_Filter_StripTags());
	
	    	//Get the input
	    	$inputText = $this->getRequest()->getParam('text', false);
	    	$inputType = $this->getRequest()->getParam('type', 1);
			
	    	//Filter the input
	    	$inputTextFiltered = $filterChain->filter($inputText);
	    	$inputType = (in_array($inputType, Mai::$allowedTextTypes)) ? $inputType : false;

	    	//Free chain
	    	unset($filterChain);
	    		
			//Check if the input is correct
	    	if(!$inputTextFiltered || $inputTextFiltered != strtolower(trim($inputText)) || !$inputType)
	    	{
				//Send the invalid input message
	    		Mai_Registry::set('output', Mai_Config::getVar()->output->invalid_input);
	    	}
	    	else
	    	{
				Mai_Log::info('TalkController: Input valid');

				switch($inputType)
				{
					case Mai::MSG_TYPE_NORMAL:
						$this->processNormal($inputTextFiltered);
						break;
						
					default:
						Mai_Registry::set('output', '-SERVER-Dit type wordt nog niet ondersteund.');
						break;
				}
	    	}
	    	
			$this->endMai();
	    }
		
    	//Send output to the view
    	$this->view->response = Mai_Registry::get('output');
    }

    /**
     * Process normal text.
     * @param $inputText
     * @return void
     */
    private function processNormal($inputText)
    {
    	//Create a new container for the input
    	$inputContainer = Mai_Container::newContainerInput();
    	
    	//Set type
    	$inputContainer->setType = Mai::MSG_TYPE_NORMAL;
    	
    	//Create a new word splitter and set the text (we save to containers)
		$wordSplitter = new Mai_Word_Splitter(Mai::STORAGE_TYPE_CLASS);
		$wordSplitter->setString($inputText);

		//Run the word splitter (and store in containers)
		$wordSplitter->run();

		//We save the sentence containers to the input container
		foreach($wordSplitter as $key => $sentence)
		{
			$inputContainer->insertSentence($sentence);
		}
		
		//Free wordSplitter
		unset($wordSplitter);
		
		//Create a new analyzer
		$wordAnalyzer = Mai_Analyzer::newWordAnalyzer(Mai::STORAGE_TYPE_CLASS);
	
		//To add the OpenTaal source we have to find the first 3 letters from every word
		$letters = $this->getFirstLettersWords($inputContainer);
		
		//Add csv source
		foreach($letters as $key => $pair)
		{
			$sourceName = 'OpenTaal-csv-'.$pair;
			$data = $this->getCsvWords($pair);
			//Add to analyzer
			$wordAnalyzer->addSource($sourceName, $data);
		}
		
		//Add other sources
		//Note: Sources ar not yet used, there is no learning system yet
		//$wordAnalyzer->addSource('local-cache', $this->getCacheWords());
		//$wordAnalyzer->addSource('user-db',		$this->getWordsFromDb());
		$wordAnalyzer->addSource('manual-cache', $this->getManualCacheWords());
		
		//Analyze each sentence
		foreach($inputContainer as $key => $sentence)
		{
			//Get all the words (by reference)
			$words = &$sentence->getAll();

			//Set the array (sentence) (by reference)
			$wordAnalyzer->set(&$words);

			//Run the analyzer
			$wordAnalyzer->run();
		}

		//Delete analyzer
		unset($wordAnalyzer);
		
		//Create a new output container object
		$outputContainer = Mai_Container::newContainerOutput();
		
		//Create new pattern analyzer
		$patternAnalyzer = Mai_Analyzer::newPatternAnalyzer();
		
		$source = array();
		$source[0]['type'] = 0;
		$source[0]['base-pattern'] = array(7, 12, 13);
		$source[0]['time-pattern'] = array(0, 1, 0);
		$source[0]['person-pattern'] = array(1, 1, 0);
		$source[0]['order-pattern'] = array(1, 2, 3);
		$source[0]['answer-base-pattern'] = array(7, 12, 13);
		$source[0]['answer-time-pattern'] = array(0, 1, 0);
		$source[0]['answer-person-pattern'] = array(3, 3, 0);
		$source[0]['answer-order-pattern'] = array(1, 2, 3);

		$source[1]['type'] = 1;
		$source[1]['base-pattern'] = array(6, 12, 7);
		$source[1]['time-pattern'] = array(0, 1, 0);
		$source[1]['person-pattern'] = array(0, 2, 1);
		$source[1]['order-pattern'] = array(1, 2, 3);
		$source[1]['answer-base-pattern'] = array(7, 12, 14);
		$source[1]['answer-time-pattern'] = array(0, 1, 0);
		$source[1]['answer-person-pattern'] = array(1, 1, 1);
		$source[1]['answer-order-pattern'] = array(0, 2, 3);
		
		//Add a 'unit-test' source
		$patternAnalyzer->addSource('unit-test', $source);
		
		foreach($inputContainer as $key => $sentence)
		{
			//Set sentence to parse
			$patternAnalyzer->set($sentence);
			
			//Run the analyzer
			$patternAnalyzer->run();
			
			//Save result to output container
			$outputContainer->insertResponsePattern($patternAnalyzer->getOutputData());
		}
		
		//Unset analyzer
		unset($patternAnalyzer);
		//Unset input container
		unset($inputContainer);

		//Create a response creator
		$response = new Mai_Response_Normal();
		
		//We need to include all verbs needed
		foreach($outputContainer as $key => $outputPattern)
		{
			foreach($outputPattern->getBasePattern() as $wordId => $wordType)
			{
				if($wordType === Mai::WT_WERKWOORD)
				{
					//Get sentence for this pattern
					$sentence = $outputPattern->getInputSentence();
					//Get the word and data
					$word = $sentence->getWord($wordId);
					//Load all data about this verb
					$response->setWTWW($word->getWwId(), $this->getCsvWordsWTWW($word->getWwId()));
				}
			}
		}
		
		//Save WTPV
		$response->setWTPV($this->getCsvWordsWTPV());
		
		//Loop through every sentence
		foreach($outputContainer as $key => $outputPattern)
		{
			//Set data to work with
			$response->set($outputPattern);
			
			//Create a response
			$response->run();
		}
    }
    
    /**
     * Start up Mai Library
     * @return void
     */
	private function startUpMai()
	{
		//Set env for Mai
		if(Zend_Registry::get('config')->application->development)
		{
			Mai::setEnv(Mai::ENV_DEVELOPMENT);
		}
		else
		{
			Mai::setEnv(Mai::ENV_PRODUCTION);
		}

		//Start up Mai library
		Mai::init();
	}

	/**
	 * End Mai library.
	 * @return void
	 */
	private function endMai()
	{
		Mai::end();
	}
	
	/**
	 * Create a new instance of Mai_Cache_Words and loads the cache file defined in config.ini
	 * @return array Words from the cache file.
	 */
	private function getCacheWords()
	{
		$cacheWords = Mai_Cache::newCacheWords(
					BASE_PATH . Zend_Registry::get('config')->cache->basewords->path,
					Zend_Registry::get('config')->cache->basewords->filename,
					Zend_Registry::get('config')->cache->basewords->varname
				);

		$words = $cacheWords->returnCacheFromFile();

		Mai_Log::info('TalkController: Loaded '.count($words).' word(s) from cache.');

		return $words;
	}

	/**
	 * Create a new instance of Mai_Cache_Words and loads the csv cache file defined in config.ini
	 * @return array Words from the cache file.
	 */
	private function getCsvWords($letterPair)
	{
		$cacheWords = Mai_Cache::newCacheWords(
					BASE_PATH . Zend_Registry::get('config')->cache->opentaal->path,
					Zend_Registry::get('config')->cache->opentaal->filename . '_' . $letterPair,
					Zend_Registry::get('config')->cache->opentaal->varname
				);

		$words = $cacheWords->returnCacheFromFile();

		Mai_Log::info('TalkController: Loaded '.count($words).' word(s) from CSV. Letter Pair: '.$letterPair);

		return $words;
	}
	
	/**
	 * Create a new instance of Mai_Cache_Words and loads the csv cache file defined in config.ini
	 * This loads only words with word type: WT_WERKWOORD
	 * @param int $wwId Verb id
	 * @return array Words from the cache file.
	 */
	private function getCsvWordsWTWW($wwId)
	{
		$cacheWords = Mai_Cache::newCacheWords(
					BASE_PATH . Zend_Registry::get('config')->cache->opentaal->path_type,
					Zend_Registry::get('config')->cache->opentaal->filename_wtww . '_' . $wwId,
					Zend_Registry::get('config')->cache->opentaal->varname
				);

		$words = $cacheWords->returnCacheFromFile();

		Mai_Log::info('TalkController: Loaded '.count($words).' word(s) from CSV WTWW '.$wwId.'.');

		return $words;
	}
	
	/**
	 * Create a new instance of Mai_Cache_Words and loads the csv cache file defined in config.ini
	 * This loads only words with word type: WT_PERSOONLIJK_VOORNAAMWOORD
	 * @return array Words from the cache file.
	 */
	private function getCsvWordsWTPV()
	{
		$cacheWords = Mai_Cache::newCacheWords(
					BASE_PATH . Zend_Registry::get('config')->cache->opentaal->path_type,
					Zend_Registry::get('config')->cache->opentaal->filename_wtpv,
					Zend_Registry::get('config')->cache->opentaal->varname
				);

		$words = $cacheWords->returnCacheFromFile();

		Mai_Log::info('TalkController: Loaded '.count($words).' word(s) from CSV WTPV.');

		return $words;
	}
	
	/**
	 * Create a new instance of Mai_Cache_Words and loads the words from the manual-cache.
	 * @return array Words from the cache file.
	 */
	private function getManualCacheWords()
	{
		$cacheWords = Mai_Cache::newCacheWords(
					BASE_PATH . Zend_Registry::get('config')->cache->manual->path,
					Zend_Registry::get('config')->cache->manual->filename,
					Zend_Registry::get('config')->cache->manual->varname
				);

			$words = $cacheWords->returnCacheFromFile();

		Mai_Log::info('TalkController: Loaded '.count($words).' word(s) from Manual Cache.');

		return $words;
	}
	

			
	/**
	 * Selects all the words from the db for the current user.
	 * @return Array Words from the db
	 */
	private function getWordsFromDb()
	{
		$query = Doctrine_Query::create(Zend_Registry::get('conn_dbuser'))
				 ->from('Word w')
				 ->where('w.user_id = ?', User_Session::returnUserId());
		$words = $query->fetchArray();

		Mai_Log::info('TalkController: Loaded '.count($words).' word(s) from the database.');

		return $words;
	}
	
	/**
	 * Selects patterns from the database.
	 * @return array
	 */
	private function getPatternsFromDb()
	{
		$query = Doctrine_Query::create(Zend_Registry::get('conn_dbuser'))
				 ->from('Word w')
				 ->where('w.user_id = ?', User_Session::returnUserId());
		$patterns = $query->fetchArray();

		Mai_Log::info('TalkController: Loaded '.count($patterns).' patterns(s) from the database.');

		return $patterns;
	}
	
	/**
	 * Returns an array with the first 3 letters of every word
	 * @param Mai_Container_Input $inputContainer
	 * @return array
	 */
	private function getFirstLettersWords(Mai_Container_Input $inputContainer)
	{
		//Container
		$firstLettersArray = array();
		
		//Loop through every word
		foreach($inputContainer->getAll() as $key => $sentence)
		{
			foreach($sentence->getAll() as $key => $word)
			{
				//Use the key 'small' for words smaller than 3 letters
				if(strlen($word->getWord()) < 3)
				{
					$letterPair = 'small';
				}
				else
				{
					$letterPair = substr($word->getWord(), 0, 3);
				}
				
				if(!in_array($letterPair, $firstLettersArray))
				{
					$firstLettersArray[] = $letterPair;
				}
			}
		}
		
		//return pairs
		return $firstLettersArray;
	}
}