<?php
/**
 * Mai
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Base
 * 
 */

/**
 * Mai is an overall class for Mai.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Base
 * 
 */

class Mai
{
	/**
	 * Text types
	 */
	const MSG_TYPE_NORMAL = 1;
	const MSG_TYPE_COMMAND = 2;
	const MSG_TYPE_LOGOUT = 3;

	/**
	 * Response types
	 */
	const RESPONSE_TYPE_NORMAL = 1;
	const RESPONSE_TYPE_COMMAND = 2;
	const RESPONSE_TYPE_LOGOUT = 3;
	
	/**
	 * Normal text types
	 */
	const MSG_NORMAL_NORMAL = 0;
	const MSG_NORMAL_QUESTION = 1;
	
    /**
     * Environments
     */
    const ENV_PRODUCTION = 1;
    const ENV_DEVELOPMENT = 2;
    const ENV_STAGING = 3;

	/**
	 * Word types (Dutch)
	 * @link http://nl.wikipedia.org/wiki/Woordsoort
	 *
	 * Note: if you change these values synchronising with Mai-Ai.org will give
	 * incorrect data. Also your database will be out of sync.
	 * 
	 *
	 */
	const WT_BIJVOEGELIJK_NAAMWOORD = 1;
	const WT_LIDWOORD = 2;
	const WT_HOOFDTELWOORD = 3;
	const WT_RANGTELWOORD = 4;
	const WT_VOEGWOORD = 5;
	const WT_VRAGEND_VOORNAAMWOORD = 6;
	const WT_PERSOONLIJK_VOORNAAMWOORD = 7;
	const WT_PERSOONLIJK_VOORNAAMWOORD_VOORWERP = 8;
	const WT_AANWIJZEND_VOORNAAMWOORD = 9;
	const WT_BEZITTELIJK_VOORNAAMWOORD = 10;
	const WT_VOORZETSEL = 11;
	const WT_WERKWOORD = 12;
	const WT_ZELFSTANDIG_NAAMWOORD = 13;
	const WT_NAAM = 14;
	const WT_BEGROETING = 15;
	const WT_AFSCHEID = 16;
	const WT_PLAATSNAAM = 17;
	
	/**
	 * Verbs (times)
	 */
	const WVT_TEGENWOORDIGE_TIJD = 1;
	const WVT_VERLEDEN_TIJD = 2;
	
	/**
	 * Word numbers (one or multiple)
	 */
	const WN_ENKELVOUD = 1;
	const WN_MEERVOUD = 2;
	
	/**
	 * Persons (ik, jij, etc.)
	 */
	const WP_1_ENKELVOUD = 1; //Ik
	const WP_1_MEERVOUD = 2; //Wij
	
	const WP_2_ENKELVOUD = 3; //Jij, u
	const WP_2_MEERVOUD = 4; //Jullie, u
	
	const WP_3_ENKELVOUD = 5; //Hij, zij, het
	const WP_3_MEERVOUD = 6; //Zij
	
	/**
	 * Analyzer response types
	 */
	const AN_RESPONSE_FULL_PATTERN = 1;
	
	/**
	 * Storage types
	 */
	const STORAGE_TYPE_ARRAY = 1;
	const STORAGE_TYPE_CLASS = 2;
	
	/**
     * @var Mai $_instance Singleton instance
     */
    protected static $_instance;
    
    /**
     * @var integer $_env The enviroment of Mai.
     */
    protected static $_env = self::ENV_PRODUCTION;
    
    /**
     * @var boolean $_autoloader Define TRUE when using an autoloader. This disables the require_once statements.
     */
    protected static $_autoloader = false;
    
    /**
     * @var string $_baseDIR Directory of the Mai class.
     */
	protected static $_baseDir;
	
	/**
	 * @var array $allowedTexTypes Allowed text types.
	 */
	public static $allowedTextTypes = array(self::MSG_TYPE_NORMAL, self::MSG_TYPE_COMMAND, self::MSG_TYPE_LOGOUT);

	/**
	 * @var array $allowedResponseTypes Allowed response types.
	 */
	public static $allowedResponseTypes = array(self::RESPONSE_TYPE_NORMAL, self::RESPONSE_TYPE_COMMAND, self::RESPONSE_TYPE_LOGOUT);
    /**
     * @var array The names of the environments
     */
   	public static $_envNames = array(
    	Mai::ENV_DEVELOPMENT 	=> 'development',
    	Mai::ENV_PRODUCTION		=> 'production',
    	Mai::ENV_STAGING		=> 'staging'
	);
    
    /**
     * @var array Allowed environments.
     */
    protected static $_envAllowed = array(self::ENV_DEVELOPMENT, self::ENV_PRODUCTION, self::ENV_STAGING);
    
    /**
     * @var array Allowed word types.
     */
    public static $allowedWordTypes = array(
    	self::WT_AANWIJZEND_VOORNAAMWOORD, self::WT_AFSCHEID, self::WT_BEGROETING,
    	self::WT_BEZITTELIJK_VOORNAAMWOORD, self::WT_BIJVOEGELIJK_NAAMWOORD, self::WT_HOOFDTELWOORD,
    	self::WT_LIDWOORD, self::WT_NAAM, self::WT_PERSOONLIJK_VOORNAAMWOORD,
    	self::WT_PERSOONLIJK_VOORNAAMWOORD_VOORWERP, self::WT_PLAATSNAAM, self::WT_RANGTELWOORD,
    	self::WT_VOEGWOORD, self::WT_VOORZETSEL, self::WT_VRAGEND_VOORNAAMWOORD,
    	self::WT_WERKWOORD, self::WT_ZELFSTANDIG_NAAMWOORD
    );
    
    /**
     * @var array Allowed times.
     */
    public static $allowedTimes = array(
    	self::WVT_TEGENWOORDIGE_TIJD,
    	self::WVT_VERLEDEN_TIJD
    );
    
    /**
     * @var array Allowed numbers (for words).
     */
    public static $allowedNumbers = array(
    	self::WN_ENKELVOUD,
    	self::WN_MEERVOUD
    );
    
    /**
     * @var array Allowed persons (for words).
     */
    public static $allowedPersons = array(
    	self::WP_1_ENKELVOUD,
    	self::WP_1_MEERVOUD,
    	self::WP_2_ENKELVOUD,
    	self::WP_2_MEERVOUD,
    	self::WP_3_ENKELVOUD,
    	self::WP_3_MEERVOUD
    );
    
    /**
     * @var Boolean Whether Mai has been setup.
     */
    protected static $_init = false;
    
	/**
	 * Private constructor to prevent creation of this object.
	 * @return void
	 */
	private function __construct(){}
	
	/**
     * Retrieve singleton instance
     * 
     * @return Mai_Config
     */
    public static function getInstance()
    {
        if (null === self::$_instance) {
        	//Start up
            self::$_instance = new self();
            //Define the location of this class
            self::$_baseDir = realpath(dirname(__FILE__));
        }
        return self::$_instance;
    }

    /**
     * Start Mai.
     * @return void
     */
    public static function init()
    {
    	//Start up the Mai class
    	self::getInstance();
    	
    	//Load required file for config
		if(!self::getAutoloader()){	require_once 'Mai/Config.php'; }
		//Let the Mai_Config class know that Mai is using it 
		Mai_Config::getInstance(self::getInstance());		
    	//Define whether the scripts uses an autoloader or not
    	if(is_bool(Mai_Config::getVar()->application->autoloader))
    	{
    		self::$_autoloader = Mai_Config::getVar()->application->autoloader;
    	}
    	
    	//Load required field for Registry
   		if(!self::getAutoloader()){	require_once 'Mai/Registry.php'; }
    	//Start registry
    	Mai_Registry::getInstance(self::getInstance());
    	
    	//Load required field for stat
    	if(!self::getAutoloader()){	require_once 'Mai/Stat.php'; }
    	
    	//Start the Mai_Stat class
		Mai_Stat::start();

		if(Mai_Config::getVar()->log->enabled)
		{
			//Start logger
			Mai_Log::init(self::$_instance);
		}
		
		Mai_Log::info("Mai: Mai started.");
		
		//Send birth date of Mai to log
		Mai_Log::info('Mai: Birth date (dd/mm/yyyy): 09-06-2008');
    	//Define innit
    	self::$_init = true;
    }
    
	/**
	 * End Mai
	 * @return void
	 */
	public static function end()
	{
		//End Mai_Stat
		Mai_Stat::end();

		Mai_Log::info('Mai: Mai ended.');
		Mai_Log::info('Mai: Execution time: '.Mai_Stat::getInstance()->executionTime());
		Mai_Log::info('Mai: Memory end: '.number_format(memory_get_usage(), 0, '.', ',').' bytes.');
		Mai_Log::info('Mai: Memory peak: '.number_format(memory_get_peak_usage(), 0, '.', ',').' bytes.');
		
		if(Mai_Config::getVar()->log->enabled && self::$_env === self::ENV_DEVELOPMENT)
		{
			//Flush log
			Mai_Log::flush();
		}
	}

    /**
     * Returns the value of $_init.
     * @return boolean
     */
    public static function isInit()
    {
    	return self::$_init;
    }
    
    /**
     * Set the environment type.
     * @param $env
     * @return void
     */
    public static function setEnv($env)
    {
		if(Mai::isInit())
		{
			trigger_error("Can't set enviroment, Mai has already been started.", E_USER_ERROR);
		}
		
    	if(!isset($env))
    	{
    		throw new UnexpectedValueException('Environment must be set.');
    	}
    	
    	if(!in_array($env, self::$_envAllowed))
    	{
    		throw new UnexpectedValueException('Environment is not valid.');
    	}
    	
    	self::$_env = $env;
    }
    
    /**
     * Return the environment type.
     * @return string
     */
    public static function getEnv()
    {
    	return self::$_env;
    }
    
    /**
     * Set whether classes are loaded using a autoloader or not.
     * @param boolean $value
     * @return void
     */
    public static function setAutoloader($value)
    {
    	if(!is_bool($value))
    	{
    		throw new UnexpectedValueException('Value can only be a boolean.');
    	}
    	
    	self::$_autoloader = $value;
    }
    
    /**
     * Return the value of $_autoloader.
     * @return boolean
     */
    public static function getAutoloader()
    {
    	return self::$_autoloader;
    }
    
    /**
     * Return the location of the Mai class.
     * @return string
     */
    public static function baseDir()
    {
    	self::getInstance();
    	
    	return self::$_baseDir;
    }
    
	/**
	 * Cloning of a singleton class is not allowed.
	 * @return void
	 */
	final public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }
}