<?php
/**
 * Mai_Container_Output
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Container
 * @subpackage Output
 * 
 */

/** Load required file */
if(!Mai::getAutoloader())
{
	require_once 'Mai/Container/Output/Interface.php';
}

/**
 * Mai_Container_Output is a container for output. It contains output sentences but also output patterns.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Container
 * @subpackage Output
 *
 */
class Mai_Container_Output implements Mai_Container_Output_Interface
{
	/**
	 * @var int Type of output.
	 */
	private $_outputType = Mai::RESPONSE_TYPE_NORMAL;
	
	/**
	 * @var array Storage for response patterns.
	 */
	private $_responsePatterns = array();

	/**
	 * 
	 * @param $creator Creator of this class
	 * @return void
	 */
	public function __construct($creator)
	{
		if(!($creator instanceof Mai_Container) && !Mai_Config::getVar()->container->allow_direct_access)
		{
			trigger_error('Please use Mai_Container to create a container class.', E_USESR_NOTICE);	
		}
	}
	
	/**
	 * Set the input type of this container.
	 * @param int $type
	 */
	public function setType($type)
	{
		//Only normal input (sentences) is currently supported
		if(!in_array($type, Mai::$allowedResponseTypes))
		{
			trigger_error('Type is not valid or not supported.', E_USER_WARNING);
		}
		
		$this->_outputType = $type;
	}
	
	/**
	 * Get the type of this container.
	 * @return int
	 */
	public function getType()
	{
		return $this->_outputType;
	}
	
	/**
	 * Insert a new response pattern
	 * @param Mai_Container_Output_Pattern $pattern
	 * @param Int $location
	 */
	public function insertResponsePattern(Mai_Container_Output_Pattern_Interface $pattern, $location = null)
	{
		if(!isset($location) || !is_numeric($location))
		{
			$this->_responsePatterns[] = $pattern;
		}
		else
		{
			$this->_responsePatterns[$location] = $pattern;
		}	
	}
	
	/**
	 * Return a response pattern from storage.
	 * @param Int $location The location (key) of storage.
	 * @param Bool $showError Set to false to return false if a sentence does not excist.
	 * @return Mai_Container_Output_Pattern
	 */
	public function getResponsePattern($location, $showError = true)
	{
		if(!isset($location) || !is_numeric($location))
		{
			trigger_error('Location not set.', E_USER_ERROR);
		}
		
		if(isset($this->_responsePatterns[$location]) && $showError === true)
		{
			trigger_error('Response pattern does not excist for location.', E_USER_NOTICE);
		}
		else
		{
			return false;
		}
		
		return $this->_responsePatterns[$location];
	}
	
	/**
	 * Function to implement the IteratorAggregate interface.
	 * @return ArrayIterator
	 */
	public function getIterator()
	{
		return new ArrayIterator($this->_responsePatterns);
	}
}