<?php
/**
 * Mai_Container_Output_Pattern
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Container
 * @subpackage Output-Pattern
 * 
 */

/** Load required file */
if(!Mai::getAutoloader())
{
	require_once 'Mai/Container/Interface.php';
}

/**
 * Mai_Container_Output_Pattern is a container class for output patterns.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Container
 * @subpackage Output-Pattern
 *
 */
class Mai_Container_Output_Pattern implements Mai_Container_Output_Pattern_Interface
{
	/**
	 * @var Mai_Container_Sentence $_inputSentence
	 */
	private $_inputSentence;
	
	/**
	 * @var array $_basePattern
	 */
	private $_basePattern = array();
	
	/**
	 * @var array $_timePattern
	 */
	private $_timePattern = array();

	/**
	 * @var array $_personPattern
	 */
	private $_personPattern = array();
	
	/**
	 * @var array $_orderPattern
	 */
	private $_orderPattern = array();
	
	/**
	 * @var int $_isQuestion
	 */
	private $_isQuestion = false;
	
	/**
	 * 
	 * @param $creator Creator of this class
	 * @param $sentence The input sentence
	 * @return void
	 */
	public function __construct($creator, Mai_Container_Sentence $sentence)
	{
		if(!($creator instanceof Mai_Container) && !Mai_Config::getVar()->container->allow_direct_access)
		{
			trigger_error('Please use Mai_Container to create a container class.', E_USESR_NOTICE);	
		}
		
		$this->_inputSentence = $sentence;
	}
	
	/**
	 * Set base pattern
	 * @param array $pattern
	 */
	public function setBasePattern(array $pattern)
	{
		$this->_basePattern = $pattern;
	}
	
	/**
	 * Get base pattern
	 * @return array
	 */
	public function getBasePattern()
	{
		return $this->_basePattern;
	}

	/**
	 * Set time pattern
	 * @param array $pattern
	 */
	public function setTimePattern(array $pattern)
	{
		$this->_timePattern = $pattern;
	}
	
	/**
	 * Get time pattern
	 * @return array
	 */
	public function getTimePattern()
	{
		return $this->_timePattern;
	}
	
	/**
	 * Set person pattern
	 * @param array $pattern
	 */
	public function setPersonPattern(array $pattern)
	{
		$this->_personPattern = $pattern;
	}
	
	/**
	 * Get person pattern
	 * @return array
	 */
	public function getPersonPattern()
	{
		return $this->_personPattern;
	}
	
	/**
	 * Get input sentence
	 * @return Mai_Container_Sentence
	 */
	public function getInputSentence()
	{
		return $this->_inputSentence;
	}
	
	/**
	 * Set order pattern
	 * @param array $pattern The input order pattern
	 * @param array $answerPattern The answer order pattern
	 */
	public function setOrderPattern(array $pattern, array $answerPattern)
	{
		$this->_orderPattern = array('pattern' => $pattern, 'answer-pattern' => $answerPattern);
	}
	
	/**
	 * Get order pattern
	 * @param string $type What order pattern to return, pattern or answer-pattern
	 */
	public function getOrderPattern($type)
	{
		if($type === 'pattern')
		{
			return $this->_orderPattern['pattern'];
		}
		elseif($type === 'answer-pattern')
		{
			return $this->_orderPattern['answer-pattern'];
		}
		else
		{
			trigger_error('Type is invalid.', E_USER_ERROR);
		}
	}
	
	/**
	 * Set this pattern to be a question
	 */
	public function setAsQuestion()
	{
		$this->_isQuestion = true;
	}
	
	/**
	 * Retrieve wheter this pattern is a question or not.
	 * @return bool
	 */
	public function isQuestion()
	{
		return $this->_isQuestion;
	}
}