<?php
/**
 * Mai_Container_Word
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Container
 * @subpackage Word
 * 
 */

/** Load required file */
if(!Mai::getAutoloader())
{
	require_once 'Mai/Container/Word/Interface.php';
}

/**
 * Mai_Container_Word is a container for a single word.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Container
 * @subpackage Sentence
 *
 */
class Mai_Container_Word implements Mai_Container_Word_Interface
{
	/**
	 * @var String The word.
	 */
	private $_word;
	/**
	 * @var Int Type of the word. (@see Mai)
	 */
	private $_type = 0;
	
	/**
	 * @var Int Time of the word. (@see Mai)
	 */
	private $_time = 0;
	
	/**
	 * @var Int Number ('meervoud'/'enkelvoud') (@see Mai)
	 */
	private $_number= 0;
	
	/**
	 * @var Int Person (@see Mai)
	 */
	private $_person = 0;
	
	/*
	 * @var Int Ww (verb) id
	 */
	private $_wwId = 0;
	
	/*
	 * @param $creator Creator of this class
	 * @param $word Word to store.
	 * @return void
	 */
	public function __construct($creator, $word)
	{
		if(!($creator instanceof Mai_Container) && !Mai_Config::getVar()->container->allow_direct_access)
		{
			trigger_error('Please use Mai_Container to create a container class.', E_USESR_NOTICE);	
		}
		
		if(!is_string($word))
		{
			trigger_error('Word must be a string', E_USER_ERROR);
		}
		
		$this->_word = $word;
	}
	/**
	 * Return the word.
	 * @return String
	 */
	public function getWord()
	{
		return $this->_word;
	}
	
	/**
	 * Return the type of the word.
	 * @return Int
	 */
	public function getType()
	{
		return $this->_type;
	}
	
	/**
	 * Return the time.
	 * @return Int
	 */
	public function getTime()
	{
		return $this->_time;
	}
	
	/**
	 * Return the number.
	 * @return Int
	 */
	public function getNumber()
	{
		return $this->_number;
	}
	
	/**
	 * Return the person.
	 * @return Int
	 */
	public function getPerson()
	{
		return $this->_person;
	}
	
	/**
	 * Set type.
	 * @return void
	 */
	public function setType($type)
	{
		if(!in_array($type, Mai::$allowedWordTypes) && $time !== 0)
		{
			trigger_error("Type is not valid.", E_USER_NOTICE);
		}
		
		$this->_type = $type;
	}
	
	/**
	 * Set time.
	 * @return void
	 */
	public function setTime($time)
	{
		if(!in_array($time, Mai::$allowedTimes) && $time !== 0)
		{
			trigger_error("Time is not valid.", E_USER_NOTICE);
		}
		
		$this->_time = $time;
	}
	
	/**
	 * Set number.
	 * @return void
	 */
	public function setNumber($number)
	{
		if(!in_array($number, Mai::$allowedNumbers) && $number !== 0)
		{
			trigger_error("Number is not valid.", E_USER_NOTICE);
		}
		
		$this->_number = $number;
	}
	
	/**
	 * Set person.
	 * @return void
	 */
	public function setPerson($person)
	{
		if(!in_array($person, Mai::$allowedPersons) && $person !== 0)
		{
			trigger_error("Person is not valid.", E_USER_NOTICE);
		}
		
		$this->_person = $person;
	}
	
	/**
	 * Set ww (verb) id
	 * @param $wwId
	 */
	public function setWwId($id)
	{
		$this->_wwId = $id;	
	}
	
	/**
	 * Get ww (verb) id.
	 * @return int.
	 */
	public function getWwId()
	{
		return $this->_wwId;
	}
}