<?php
/**
 * Mai_Response_Normal
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Response
 * @subpackage Normal
 * 
 */

/**
 * Mai_Response_Normal creates responses from diffrent sources.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @category Mai
 * @package Response
 * @subpackage Normal
 * 
 */

class Mai_Response_Normal
{
	/*
	 * @var Mai_Container_Output_Pattern $_pattern
	 */
	private $_pattern;
	
	/**
	 * @var array $_sourceWTPV
	 */
	private $_sourceWTPV = array();
	
	/**
	 * @var array $_sourceWTWW
	 */
	private $_sourceWTWW = array();
	
	/**
	 * Set source.
	 * @param Mai_Container_Output_Pattern_Interface $pattern
	 * @return void
	 */
	public function set(Mai_Container_Output_Pattern_Interface $pattern)
	{
		$this->_pattern = $pattern;
	}
	
	/**
	 * Set data with word type: WT_PERSOONLIJK_VOORNAAMWOORD
	 * @param array $data
	 * @return void
	 */
	public function setWTPV(array $data)
	{
		$this->_sourceWTPV = $data;
	}

	/**
	 * Set data with word type: WT_WERKWOORD
	 * @param array $data
	 * @return void
	 */
	public function setWTWW($wwId, array $data)
	{
		$this->_sourceWTWW[$wwId] = $data;
	}
	
	/**
	 * Create a response from source set by set()
	 * @return void
	 */
	public function run()
	{
		if(!isset($this->_pattern))
		{
			trigger_error('No source set. Cannot run.', E_USER_WARNING);
		}
		
		//Temporary
		$output = '';
		
		//Get time
		$timePattern = $this->_pattern->getTimePattern();
		//Get person
		$personPattern = $this->_pattern->getPersonPattern();
		//Get input order
		$inputOrder = $this->_pattern->getOrderPattern('pattern');
		//Get answer order
		$answerOrder = $this->_pattern->getOrderPattern('answer-pattern');
			
		//Loop through the words
		foreach($this->_pattern->getBasePattern() as $key => $wordType)
		{
			//Get current word from sentence
			$word = $this->_pattern->getInputSentence()->getWord($key);
			
			//Get person for this word
			$person = $personPattern[$key];
			//Get time for this word
			$time = $timePattern[$key];

			$responseWord = ''; //Store word
			
			//Actions based on word type
			switch($wordType)
			{
				case Mai::WT_WERKWOORD:
					//The word is a verb, we have to transform the time and person
					//First look for other verbs
					if(!isset($this->_sourceWTWW[$word->getWwId()]))
					{
						//Nothing to do
						//Verb unknown!
						//@Mai Todo add support for unkown verbs
						die('-SERVER-Onbkend werkwoord.');
					}
					
					//Check the right word
					foreach($this->_sourceWTWW[$word->getWwId()] as $key => $word)
					{
						if($word['person'] === $person && $word['time'] == $time)
						{
							$responseWord = $word['word'];
							//Break from loop
							break;
						}
					}
					break;
				case Mai::WT_PERSOONLIJK_VOORNAAMWOORD:
					//Find the right version
					foreach($this->_sourceWTPV as $key => $word)
					{
						if($word['person'] === $person && $word['time'] == $time)
						{
							$responseWord = $word['word'];
							//Break from loop
							break;
						}
					}
					break;
				case Mai::WT_PERSOONLIJK_VOORNAAMWOORD_VOORWERP:
					
					break;
					
				case Mai::WT_ZELFSTANDIG_NAAMWOORD:
					$responseWord = $word->getWord();
					break;
			}
			$output .= $responseWord . ' ';
		}
		
		die($output);
	}
}