<?php
/**
 * CSV Parser
 * 
 * Copyright (C) 2009  Mai (Me Artificial Intelligence)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Wouter Bulten (wouterbulten@mai-ai.org)
 * @copyright Copyright (C) 2009  Mai (Me Artificial Intelligence)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @link http://www.launchpad.net/mai
 * 
 * @package Other
 * 
 */

//Setup global settings
error_reporting(E_ALL | E_STRICT);
ini_set('display_startup_errors', 1);  
ini_set('display_errors', 1);

//Enlarge memory
ini_set('memory_limit', '230M');

//Path seperator for include paths (Mac OS: ':', Windows ';')
define('PATH_SEPERATOR', ':');

//Application path for include paths
define('BASE_PATH', realpath(dirname(__FILE__) . '/../'));
define('APPLICATION_PATH', BASE_PATH . '/application');

//Timezone
date_default_timezone_set('Europe/Amsterdam');

//Setup include paths
set_include_path('.'
    . PATH_SEPERATOR . get_include_path()
    
    //Include paths for libraries
    . PATH_SEPERATOR . BASE_PATH . '/library'
    
    //Models
    . PATH_SEPERATOR . APPLICATION_PATH . '/models'
);
    	
//Setup autoloading

//Require files
require_once('Zend/Loader.php');
require_once('Zend/Loader/Autoloader.php');

$autoloader = Zend_Loader_Autoloader::getInstance();

//Register namespace Mai_
$autoloader->registerNamespace('Mai_');

$autoloader->setFallbackAutoloader(true);

//Show errors
$autoloader->suppressNotFoundWarnings(false);

//Start Mai because we need Mai Lib
Mai::init();

//Open file
$file = fopen(BASE_PATH . '/data/csv/words_relations.csv', "r");

//Counters
$number = 0;
$skip = 0;
$letterCounter = array(); //Amount of words for each letter pair

//Parse
while($data = fgetcsv($file, 0, ';', '"'))
{	
	//Container of the data and word
	//Set some base values
	$container = array('person' => 0, 'time' => 0, 'ww_id' => 0);
	//echo a character to show the script is running
	echo '|';
	
	//Get type
	$type = substr($data[1], 0, 2);
	//Get extra info (after type)
	$extra = substr($data[1], 2);
	
	//Get the type, time and form
	switch($type)
	{
		case 'AJ': //Bijvoegelijk naamwoord
			$container['wordtype'] = Mai::WT_BIJVOEGELIJK_NAAMWOORD;		
			break;
		case 'NN': //Zelfstandig naamwoord
		case 'Nn':
			$container['wordtype'] = Mai::WT_ZELFSTANDIG_NAAMWOORD;
			if($extra{0} == '1')
			{
				$container['number'] = Mai::WN_ENKELVOUD;
			}
			elseif($extra{0} == '2')
			{
				$container['number'] = Mai::WN_MEERVOUD;
			}
			else
			{
				//Set to capture all words
				$container['number'] = Mai::WN_ENKELVOUD;;
			}
			break;
		
		case 'NM':
			if($extra == 'r')
			{
				$container['wordtype'] = Mai::WT_RANGTELWOORD;
			}
			else
			{
				$container['wordtype'] = Mai::WT_HOOFDTELWOORD;
			}
			break;
		
		case 'PV': //Persoonlijk voornaamwoord, bezittelijk voornaamwoord
			//Increase amount to give priority
			$container['amount'] = $data[4] + 2;
			switch($extra)
			{
				case '1':
					$container['wordtype'] = Mai::WT_PERSOONLIJK_VOORNAAMWOORD;
					$container['person'] = Mai::WP_1_ENKELVOUD;
					break;
				
				case '1a':
					$container['wordtype'] = Mai::WT_PERSOONLIJK_VOORNAAMWOORD_VOORWERP;
					$container['person'] = Mai::WP_1_ENKELVOUD;
					break;
				
				case '1b':
					$container['wordtype'] = Mai::WT_BEZITTELIJK_VOORNAAMWOORD;
					$container['person'] = Mai::WP_1_ENKELVOUD;
					break;

				case '2':
					$container['wordtype'] = Mai::WT_PERSOONLIJK_VOORNAAMWOORD;
					$container['person'] = Mai::WP_2_ENKELVOUD;
					break;
				
				case '2a':
					$container['wordtype'] = Mai::WT_PERSOONLIJK_VOORNAAMWOORD_VOORWERP;
					$container['person'] = Mai::WP_2_ENKELVOUD;
					break;
				
				case '2b':
					$container['wordtype'] = Mai::WT_BEZITTELIJK_VOORNAAMWOORD;
					$container['person'] = Mai::WP_2_ENKELVOUD;
					break;
				
				case '3':
					$container['wordtype'] = Mai::WT_PERSOONLIJK_VOORNAAMWOORD;
					$container['person'] = Mai::WP_3_ENKELVOUD;
					break;
				
				case '3a':
					$container['wordtype'] = Mai::WT_PERSOONLIJK_VOORNAAMWOORD_VOORWERP;
					$container['person'] = Mai::WP_3_ENKELVOUD;
					break;
				
				case '3b':
					$container['wordtype'] = Mai::WT_BEZITTELIJK_VOORNAAMWOORD;
					$container['person'] = Mai::WP_3_ENKELVOUD;
					break;
				
				case 'pl':
					$container['wordtype'] = Mai::WT_PERSOONLIJK_VOORNAAMWOORD;
					$container['person'] = Mai::WP_1_MEERVOUD;
					break;
				
				case 'pla':
					$container['wordtype'] = Mai::WT_PERSOONLIJK_VOORNAAMWOORD_VOORWERP;
					$container['person'] = Mai::WP_1_MEERVOUD;
					break;
								
				case 'plb':
					$container['wordtype'] = Mai::WT_BEZITTELIJK_VOORNAAMWOORD;
					$container['person'] = Mai::WP_1_MEERVOUD;
					break;
				
				default:
					//Error
					unset($type);
					break;
			}
			break;
			
		case 'VB': //Werkwoord
			$container['wordtype'] = Mai::WT_WERKWOORD;

			//Increase amount to give priority
			$container['amount'] = $data[4] + 1;
			
			switch($extra)
			{
				case '1':
					$container['person'] = Mai::WP_1_ENKELVOUD;
					$container['time'] = Mai::WVT_TEGENWOORDIGE_TIJD;
					break;
				case '2':
					$container['person'] = Mai::WP_2_ENKELVOUD;
					$container['time'] = Mai::WVT_TEGENWOORDIGE_TIJD;
					break;
				case '3':
					$container['person'] = Mai::WP_3_ENKELVOUD;
					$container['time'] = Mai::WVT_TEGENWOORDIGE_TIJD;
					break;
				
				case 'h':
					$container['person'] = Mai::WP_1_MEERVOUD;
					$container['time'] = Mai::WVT_VERLEDEN_TIJD;
					break;
					
				case 'h1':
					$container['person'] = Mai::WP_1_ENKELVOUD;
					$container['time'] = Mai::WVT_VERLEDEN_TIJD;
					break;
					
				case 'i':
					$container['person'] = Mai::WP_1_MEERVOUD;
					$container['time'] = Mai::WVT_TEGENWOORDIGE_TIJD;
					break;
			}
			
			$container['ww_id'] = $data[3];
			break;
			
		case 'PN': //Plaatsnaam
		case 'DT':
			$container['wordtype'] = Mai::WT_PLAATSNAAM;
		break;
		
		case 'PR': //Voorzetsel
			$container['wordtype'] = Mai::WT_VOORZETSEL;
			break;
		
		//No-save
		case 'LW':
		default:
			//Don't save these types
			unset($type);
			$skip++;
		break;
	}
	
	//Only add when data is available
	if(isset($type) && $type != '')
	{
		$container['word'] = addslashes($data[0]);
		
		if(!isset($container['amount']))
		{
			$container['amount'] = $data[4];
		}
		
		//Set data that has not been set
		if(!isset($container['number']))
		{
			$container['number'] = 0;
		}
		if(!isset($container['person']))
		{
			$container['person'] = 0;
		}
			if(!isset($container['time']))
		{
			$container['time'] = 0;
		}
		
		$number++;
		
		//We store based on the first three letters
		//if the word is smaller or equal to 2 letters we store it seperately
		if(strlen($data[0]) < 3)
		{
			$wordStorage['small'][] = $container;
		}
		else
		{
			//Get first 3 letters
			$letter = strtolower(substr($data[0], 0, 3));
			
			//Used to fix a strange bug.
			//The parser triggerd an error with the letter pair km/
			if($letter == 'km/')
			{
				echo 'Skipped: '.$data[0];
			}
			else
			{
				//Store
				$wordStorage[$letter][] = $container;
				
				if(isset($letterCounter[$letter]))
				{
					$letterCounter[$letter]++;
				}
				else
				{
					$letterCounter[$letter] = 1;
				}
			}
		}
	}
}

//Sort counter
asort($letterCounter, SORT_NUMERIC);

//Create cache files
foreach($wordStorage as $letter => $words)
{
	$usageCount = array();
	
	//Sort words by usage count
	foreach($words as $key => $value)
	{
		//Create a array with only the count for the sorter
		$usageCount[$key] = $value['amount'];
	}

	//Sort
	array_multisort($usageCount, SORT_DESC, $words);
	
	$cache = Mai_Cache::newCacheWords(BASE_PATH . '/cache/opentaal/', 'opentaalCache_'.$letter, 'cache_Opentaal');
	$cache->createCache($words);
}


//End Mai
Mai::end();

echo $number .' words parsed in '.round(Mai_Stat::getInstance()->executionTime()).' seconds. '.$skip.' words skipped.';